/**
 * @license
 * Copyright 2019 Google LLC. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =============================================================================
 */
/// <amd-module name="@tensorflow/tfjs-backend-webgpu/dist/backend_webgpu" />
/// <reference types="@webgpu/types/dist" />
import './flags_webgpu';
import { backend_util, DataStorage, DataToGPUWebGPUOption, DataType, GPUData, KernelBackend, Rank, RecursiveArray, TensorBuffer, TensorInfo, TimingInfo } from '@tensorflow/tfjs-core';
import { BufferManager } from './buffer_manager';
import { TextureManager } from './texture_manager';
import * as webgpu_program from './webgpu_program';
export interface WebGPUMemoryInfo extends backend_util.MemoryInfo {
    numBytesInGPU: number;
    numBytesAllocatedInGPU: number;
    unreliable: boolean;
}
declare type BufferInfo = {
    byteSize: number;
    usage: GPUBufferUsageFlags;
    buffer?: GPUBuffer;
};
declare type TensorBufferInfo = {
    values: backend_util.BackendValues;
    dtype: DataType;
    shape: number[];
    bufferInfo: BufferInfo;
    refCount: number;
    complexTensorInfos?: {
        real: TensorInfo;
        imag: TensorInfo;
    };
};
declare type ExternalImage = HTMLCanvasElement | ImageBitmap | OffscreenCanvas;
interface DataId {
}
export declare type WebGPUKernelInfo = {
    name: string;
    query: Promise<number>;
};
export declare type TimerNode = RecursiveArray<WebGPUKernelInfo> | WebGPUKernelInfo;
export interface WebGPUTimingInfo extends TimingInfo {
    uploadWaitMs: number;
    downloadWaitMs: number;
}
declare type ProgramUniform = Array<{
    type: string;
    data: number[];
}>;
export declare class WebGPUBackend extends KernelBackend {
    device: GPUDevice;
    queue: GPUQueue;
    currentCommandEncoder: GPUCommandEncoder;
    currentComputePass: GPUComputePassEncoder;
    tensorMap: DataStorage<TensorBufferInfo>;
    supportTimeQuery: boolean;
    dummyCanvas: HTMLCanvasElement;
    dummyContext: GPUCanvasContext;
    private static nextDataId;
    private nextDataId;
    private commandQueueOwnedIds;
    private layoutCache;
    private pipelineCache;
    private bufferManager;
    private textureManager;
    private tensorDisposalQueue;
    private uniformDisposalQueue;
    private stagingDisposalQueue;
    private textureDisposalQueue;
    private disposed;
    private programTimersStack;
    private activeTimers;
    private uploadWaitMs;
    private downloadWaitMs;
    private dispatchNumberInEncoder;
    private querySet;
    private fromPixelTextureLayout;
    private fromPixelImportTextureLayout;
    constructor(device: GPUDevice, supportTimeQuery?: boolean);
    floatPrecision(): 32;
    defaultGpuBufferUsage(): number;
    flushDisposalQueue(): void;
    /**
     * Dispose the memory if the dataId has 0 refCount. Return true if the memory
     * is released or memory is not managed in this backend, false if memory is
     * not cleared.
     * @param dataId
     * @oaram force Optional, remove the data regardless of refCount
     */
    disposeData(dataId: DataId, force?: boolean): boolean;
    memory(): WebGPUMemoryInfo;
    getBufferManager(): BufferManager;
    getTextureManager(): TextureManager;
    acquireBuffer(byteSize: number, usage?: GPUBufferUsageFlags): GPUBuffer;
    maybeReleaseBuffer(dataId: DataId): void;
    /** Return refCount of a `TensorData`. */
    refCount(dataId: DataId): number;
    /** Increase refCount of a `TensorData`. */
    incRef(dataId: DataId): void;
    /** Decrease refCount of a `TensorData`. */
    decRef(dataId: DataId): void;
    write(values: backend_util.BackendValues, shape: number[], dtype: DataType): DataId;
    move(dataId: DataId, values: backend_util.BackendValues, shape: number[], dtype: DataType, refCount: number): void;
    submitQueue(): void;
    getBuffer(dataId: DataId): GPUBuffer;
    ensureCommandEncoderReady(): void;
    ensureComputePassEnded(): void;
    getComputePass(): GPUComputePassEncoder;
    getBufferData(buffer: GPUBuffer, size: number): Promise<backend_util.BackendValues>;
    private convertAndCacheOnCPU;
    readSync(dataId: object): backend_util.BackendValues;
    read(dataId: object): Promise<backend_util.BackendValues>;
    /**
     * Read tensor to a new GPUBuffer.
     * @param dataId The source tensor.
     * @param options
     *     customBufShape: Optional. If set, will use the user defined buffer
     *     size to create the buffer.
     */
    readToGPU(dataId: DataId, options?: DataToGPUWebGPUOption): GPUData;
    bufferSync<R extends Rank, D extends DataType>(t: TensorInfo): TensorBuffer<R, D>;
    time(f: () => void): Promise<WebGPUTimingInfo>;
    getAndSavePipeline(key: string, getPipeline: () => GPUComputePipeline): GPUComputePipeline;
    makeTensorInfo(shape: number[], dtype: DataType, values?: backend_util.BackendValues | string[]): TensorInfo;
    private tensorToBinding;
    getQueryTime(query: GPUQuerySet): Promise<number>;
    uploadToGPU(dataId: DataId): void;
    private makeUniforms;
    private createLayout;
    private getCachedOrCreateLayout;
    runWebGPUProgram(program: webgpu_program.WebGPUProgram, inputs: TensorInfo[], outputDtype: DataType, programUniforms?: ProgramUniform, output?: TensorInfo): TensorInfo;
    private getFromPixelTextureLayout;
    private createFromPixelTextureLayout;
    private copyExternalImageToTexture;
    runFromPixelsProgram(program: webgpu_program.WebGPUProgram, outShape: number[], programUniforms: ProgramUniform, useImport: boolean, externalImage: ExternalImage | HTMLVideoElement): TensorInfo;
    getTimeFromQuerySet(querySet: GPUQuerySet): Promise<number>;
    shouldExecuteOnCPU(inputs: TensorInfo[], sizeThreshold?: number): boolean;
    numDataIds(): number;
    dispose(): void;
}
export {};
